/*
 *   IBM Linux Device Font Library
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Author: Julie Zhuo
 *   Sept. 25, 2002
 *   $Id: MyDOMErrorHandler.cpp,v 1.1 2002/11/25 22:44:37 jzhuo Exp $
 */

#include <iostream>
#include <string>

#include "MyDOMErrorHandler.hpp"

#include <xercesc/util/XMLString.hpp>

using namespace std;
using namespace DevFont;

MyDOMErrorHandler::MyDOMErrorHandler() :

  fSawErrors(false)
{
}

MyDOMErrorHandler::~MyDOMErrorHandler()
{
}


// ---------------------------------------------------------------------------
//  MyDOMHandlers: Overrides of the DOM ErrorHandler interface
// ---------------------------------------------------------------------------
bool MyDOMErrorHandler::handleError(const DOMError& domError)
{
  fSawErrors = true;
  if (domError.getSeverity() == DOMError::DOM_SEVERITY_WARNING)
    cerr << "\nWarning at file ";
  else if (domError.getSeverity() == DOMError::DOM_SEVERITY_ERROR)
    cerr << "\nError at file ";
  else
    cerr << "\nFatal Error at file ";

  char * message = XMLString::transcode(domError.getMessage());

  cerr << domError.getLocation()->getURI()
       << ", line " << domError.getLocation()->getLineNumber()
       << ", char " << domError.getLocation()->getColumnNumber()
       << "\n  Message: " << string(message) << endl;

  delete [] message;

  return false;
}

void MyDOMErrorHandler::resetErrors()
{
  fSawErrors = false;
}

