/*
 *   IBM Linux Device Font Library
 *   Copyright (c) International Business Machines Corp., 2002
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   Author: David L. Wagner
 *   October 16, 2002
 *   $Id: XmlFile.hpp,v 1.2 2002/12/06 00:16:52 jzhuo Exp $
 */

#ifndef _XmlFile
#define _XmlFile

#include <iostream>


class DOMBuilder;
class DOMElement;
class DOMNode;

namespace DevFont
{

class XmlFile
{
public:
  /** Construct the XML file from the given path. */
  explicit XmlFile (const char *path);

  /** Delete the parser when we are done. */
  ~XmlFile();

  /** Find an Element with the given name starting at the given Node. */
  static DOMElement *findElement (const DOMNode *, const char *);

  /** Find the first Element with the given name. */
  DOMElement *findElement (const char *) const;

  /** Return the root Element of the file. */
  const DOMElement *getRootElement () const { return rootElement; }

  /** The UPDF file name */
  char* fileName;

private:
  /** Whether we have initialized the parser. */
  static bool initialized;
  
  /** The static initializer code. */
  static bool initialize();

  /** The parser. */
  DOMBuilder *parser;

  /** The root of the DOM tree. */
  DOMElement *rootElement;

};

  /** Print out the xml file name */
  std::ostream & operator<< (std::ostream &, const XmlFile *);

}

#endif
