/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifdef INCLUDE_JP_UPDF_BOOKLET
#include <UPDFDeviceBooklet.hpp>
#endif
//#include <UPDFDeviceCollation.hpp>
#ifdef INCLUDE_JP_COMMON_COPIES
#include <UPDFDeviceCopies.hpp>
#endif
//#include <UPDFDeviceDestination.hpp>
#ifdef INCLUDE_JP_COMMON_SIDE
#include <UPDFDeviceSide.hpp>
#endif
#ifdef INCLUDE_JP_UPDF_JOGGING
#include <UPDFDeviceJogging.hpp>
#endif
#include <UPDFDeviceForm.hpp>
#include <UPDFDeviceMedia.hpp>
#ifdef INCLUDE_JP_COMMON_NUP
#include <UPDFDeviceNUp.hpp>
#endif
#include <UPDFDeviceOrientation.hpp>
#include <UPDFDevicePrintMode.hpp>
#include <UPDFDeviceResolution.hpp>
#ifdef INCLUDE_JP_COMMON_SCALING
#include <UPDFDeviceScaling.hpp>
#endif
#ifdef INCLUDE_JP_COMMON_STITCHING
#include <UPDFDeviceStitching.hpp>
#endif
#include <UPDFDeviceTray.hpp>
#ifdef INCLUDE_JP_COMMON_TRIMMING
#include <UPDFDeviceTrimming.hpp>
#endif
//#include <UPDFDeviceCommands.hpp>
#include <UPDFDeviceInstance.hpp>
#include <UPDFDeviceBlitter.hpp>
#include <UPDFDevice.hpp>
#include <JobProperties.hpp>

#include <glob.h>

static bool      isDeviceConfigurationXMLFile       (xmlDocPtr   doc);
static xmlDocPtr getDeviceConfigurationFromFile     (PSZCRO      pszMasterXMLFile);
static xmlDocPtr getDefaultDeviceConfiguration      ();
static xmlDocPtr getUDRFromDeviceConfiguration      (xmlDocPtr   docDC);
static bool      getOmniNames                       (xmlDocPtr   docDC,
                                                     PSZRO      *ppszDriverName,
                                                     PSZRO      *ppszDeviceName,
                                                     PSZRO      *ppszShortName);
static void      MyValidityErrorFunc                (void       *ctx,
                                                     PSZCRO      msg,
                                                                 ...);
static void      MyValidityWarningFunc              (void       *ctx,
                                                     PSZCRO      msg,
                                                                 ...);

PSZCRO
getVersion ()
{
   return VERSION;
}

Enumeration *
getDeviceEnumeration (PSZCRO pszLibraryName,
                      bool   fBuildOnly)
{
   // @TBD
   return 0;
}

Device *
newDeviceW_Advanced (bool fAdvanced)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": fAdvanced = " << fAdvanced << std::endl;
#endif

   xmlDocPtr docDC         = getDefaultDeviceConfiguration ();
   xmlDocPtr docUDR        = 0;
   PSZRO     pszDriverName = 0;
   PSZRO     pszDeviceName = 0;
   PSZRO     pszShortName  = 0;

   if (docDC)
   {
      docUDR = getUDRFromDeviceConfiguration (docDC);
   }

   if (  docDC
      && docUDR
      && getOmniNames (docUDR,
                       &pszDriverName,
                       &pszDeviceName,
                       &pszShortName)
      )
   {
      PrintDevice *pPrintDevice = new UPDFDevice (docDC,
                                                  docUDR,
                                                  pszDriverName,
                                                  pszDeviceName,
                                                  pszShortName,
                                                  0,  // @TBD
                                                  0); // @TBD

      if (pPrintDevice)
      {
         pPrintDevice->initialize ();

         if (fAdvanced)
            return pPrintDevice;
         else
            return new OmniProxy (pPrintDevice);
      }
   }

   return 0;
}

Device *
newDeviceW_JopProp_Advanced (PSZCRO pszJobProperties,
                             bool   fAdvanced)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": pszJobProperties = \"" << pszJobProperties << "\", fAdvanced = " << fAdvanced << std::endl;
#endif

   xmlDocPtr  docDC         = 0;
   xmlDocPtr  docUDR        = 0;
   PSZRO      pszDriverName = 0;
   PSZRO      pszDeviceName = 0;
   PSZRO      pszShortName  = 0;

   if (  pszJobProperties
      && *pszJobProperties
      )
   {
      JobProperties          jobProp (pszJobProperties);
      JobPropertyEnumerator *pEnum                      = 0;

      pEnum = jobProp.getEnumeration ();

      while (pEnum->hasMoreElements ())
      {
         PSZCRO pszKey   = pEnum->getCurrentKey ();
         PSZCRO pszValue = pEnum->getCurrentValue ();

         if (0 == strcmp (pszKey, "UPDFMasterFile"))
         {
            docDC = getDeviceConfigurationFromFile (pszValue);

            if (!docDC)
            {
               DebugOutput::getErrorStream () << "Warning: Could not load Device Configuration from " << pszValue << std::endl;
            }
         }
#ifndef RETAIL
         else if (0 == strcmp (pszKey, "debugoutput"))
         {
            DebugOutput::setDebugOutput (pszValue);
         }
#endif
         pEnum->nextElement ();
      }

      delete pEnum;
   }

   if (!docDC)
   {
      docDC = getDefaultDeviceConfiguration ();
   }

   if (docDC)
   {
      docUDR = getUDRFromDeviceConfiguration (docDC);
   }

   if (  docDC
      && docUDR
      && getOmniNames (docUDR,
                       &pszDriverName,
                       &pszDeviceName,
                       &pszShortName)
      )
   {
      PrintDevice *pPrintDevice = new UPDFDevice (docDC,
                                                  docUDR,
                                                  pszDriverName,
                                                  pszDeviceName,
                                                  pszShortName,
                                                  pszJobProperties,
                                                  0);               // @TBD

      pPrintDevice->initialize ();

      if (fAdvanced)
         return pPrintDevice;
      else
         return new OmniProxy (pPrintDevice);
   }

   if (docDC)
   {
      xmlFreeDoc (docDC);
   }

   return 0;
}

void
deleteDevice (Device *pDevice)
{
   delete pDevice;
}

static bool
isDeviceConfigurationXMLFile (xmlDocPtr doc)
{
   xmlNodePtr nodeRoot = xmlDocGetRootElement (doc);
   bool       fRc      = false;

   if (nodeRoot)
   {
      xmlNodePtr nodeElm = 0;

      nodeElm = firstNode (nodeRoot);

      if (nodeElm)
      {
         if (0 == strcmp ((PSZCRO)nodeElm->name, "DeviceConfiguration"))
         {
            fRc = true;
         }
      }
   }

   return fRc;
}

static xmlDocPtr
getDeviceConfigurationFromFile (PSZCRO pszMasterXMLFile)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": pszMasterXMLFile = " << pszMasterXMLFile << std::endl;
#endif

   xmlDocPtr  docRet   = 0;

   docRet = xmlParseFile (pszMasterXMLFile);
///xmlValidateDocument (&validator, docRet); // @TBD

   if (docRet)
   {
      if (isDeviceConfigurationXMLFile (docRet))
      {
         return docRet;
      }

      xmlFreeDoc (docRet);
   }

   return 0;
}

static xmlDocPtr
getDefaultDeviceConfiguration ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": " << std::endl;
#endif

   xmlDocPtr docRet  = 0;
   glob_t    globbuf;
   int       rc;

   memset (&globbuf, 0, sizeof (globbuf));
   rc = glob ("*.xml", 0, NULL, &globbuf);

   if (0 == rc)
   {
      for (int i = 0; i < (int)globbuf.gl_pathc; i++)
      {
         docRet = xmlParseFile (globbuf.gl_pathv[i]);
/////////xmlValidateDocument (&validator, docRet); // @TBD

         if (docRet)
         {
            if (isDeviceConfigurationXMLFile (docRet))
            {
               break;
            }

            xmlFreeDoc (docRet);
         }
      }
   }

   globfree (&globbuf);

   return docRet;
}

static xmlDocPtr
getUDRFromDeviceConfiguration (xmlDocPtr docDC)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": " << std::endl;
#endif

   xmlDocPtr   docRet     = 0;
   xmlNodePtr  nodeRootDC = 0;
   PSZRO       pszUDR     = 0;

   nodeRootDC = xmlDocGetRootElement (docDC);

   pszUDR = (PSZRO)xmlGetProp (nodeRootDC,
                               (xmlChar *)"Unit_Description_Reference");

   if (pszUDR)
   {
      std::string stringUDR = pszUDR;

      stringUDR += ".xml";

      docRet = xmlParseFile (stringUDR.c_str ());

      if (docRet)
      {
/////////xmlValidateDocument (&validator, docRet); // @TBD
      }

      free ((void *)pszUDR);
   }

   return docRet;
}

static bool
getOmniNames (xmlDocPtr   docDC,
              PSZRO      *ppszDriverName,
              PSZRO      *ppszDeviceName,
              PSZRO      *ppszShortName)
{
   bool fRc = false;

   if (  docDC
      && ppszDriverName
      )
   {
      xmlNodePtr nodeDeviceHeader = xmlDocGetRootElement (docDC);

      nodeDeviceHeader = UPDFDevice::findEntry (nodeDeviceHeader,
                                                "Device_Header",
                                                DebugOutput::shouldOutputUPDFDevice ());

      if (nodeDeviceHeader)
      {
         *ppszDriverName = (PSZRO)xmlGetProp (nodeDeviceHeader,
                                              (xmlChar *)"Manufacturer");

         *ppszDeviceName = (PSZRO)xmlGetProp (nodeDeviceHeader,
                                              (xmlChar *)"ProductName");

         if (  *ppszDeviceName
            && 0 < strlen (*ppszDeviceName)
            )
         {
            *ppszShortName = (PSZRO)malloc (strlen (*ppszDeviceName) + 1);

            if (*ppszShortName)
            {
               PSZRO  pszFrom = *ppszDeviceName;
               PSZ    pszTo   = (PSZ)*ppszShortName;

               while (*pszFrom)
               {
                  switch (*pszFrom)
                  {
                  case '-':
                  case '+':
                  case '(':
                  case ')':
                  case ' ':
                  case '/':
                  case '\\':
                  {
                     *pszTo++ = '_';
                     pszFrom++;
                     break;
                  }
                  default:
                  {
                     *pszTo++ = *pszFrom++;
                  }
                  }
               }
               *pszTo++ = '\0';
            }
         }

         if (  *ppszDriverName
            && *ppszDeviceName
            && *ppszShortName
            )
         {
            fRc = true;
         }
      }
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ())
      DebugOutput::getErrorStream () << "UPDFDevice: "
                                     << __FUNCTION__
                                     << ": *ppszDriverName = "
                                     << (ppszDriverName ? *ppszDriverName : "(null)")
                                     << ", *ppszDeviceName = "
                                     << (ppszDeviceName ? *ppszDeviceName : "(null)")
                                     << ", *ppszShortName = "
                                     << (ppszShortName ? *ppszShortName : "(null)")
                                     << std::endl;
#endif

   return fRc;
}

static void
MyValidityErrorFunc (void *ctx, PSZCRO msg, ...)
{
   va_list list;

   va_start (list, msg);

   printf ("Error: ");
   vprintf (msg, list);

   va_end (list);

   if (ctx)
   {
      // @TBD
   }
}

static void
MyValidityWarningFunc (void *ctx, PSZCRO msg, ...)
{
   va_list list;

   va_start (list, msg);

   printf ("Warning: ");
   vprintf (msg, list);

   va_end (list);
}

void UPDFDevice::
commonInit (xmlDocPtr docDC,
            xmlDocPtr docUDR)
{
   xmlValidCtxt validator = {
      this,
      MyValidityErrorFunc,
      MyValidityWarningFunc
   };

   setCapabilities ( Capability::MIRROR
                   | Capability::MONOCHROME
                   );

   setRasterCapabilities ( TOP_TO_BOTTOM
                         );

   docDC_d  = docDC;
   docUDR_d = docUDR;

   if (  docDC_d
      && docUDR_d
      )
   {
      nodeRootDC_d  = xmlDocGetRootElement (docDC_d);
      nodeRootUDR_d = xmlDocGetRootElement (docUDR_d);

      xmlNodePtr nodeLocale = 0;

      if ((nodeLocale = findEntry (nodeRootDC_d, "Locale", DebugOutput::shouldOutputUPDFDevice ())) != 0)
      {
         PSZCRO pszLocale = (PSZCRO)xmlGetProp (nodeLocale, (xmlChar *)"Locale_Reference");

         if (pszLocale)
         {
            std::string stringLocale = pszLocale;

            stringLocale += ".xml";

            docLocale_d = xmlParseFile (stringLocale.c_str ());

            if (docLocale_d)
            {
               xmlValidateDocument (&validator, docLocale_d);

               nodeRootLocale_d = xmlDocGetRootElement (docLocale_d);
            }

            free ((void *)pszLocale);
         }
      }

      PSZCRO pszCS = (PSZCRO)xmlGetProp (nodeRootDC_d, (xmlChar *)"CommandSequences_Reference");

      if (pszCS)
      {
         std::string stringCS = pszCS;

         stringCS += ".xml";

         docCS_d = xmlParseFile (stringCS.c_str ());

         if (docCS_d)
         {
            xmlValidateDocument (&validator, docCS_d);

            nodeRootCS_d = xmlDocGetRootElement (docCS_d);
         }

         free ((void *)pszCS);
      }
   }

   pInstance_d = new UPDFDeviceInstance (this);

   setDeviceInstance (pInstance_d);
   setDeviceBlitter (new UPDFDeviceBlitter (this));

   setPDL (new PDL (PDL::PDL_Epson,PDL::LEVEL_ESCP_2,1,0)); // @TBD

   setVirtualUnits ();
}

UPDFDevice::
UPDFDevice (xmlDocPtr  docDC,
            xmlDocPtr  docUDR,
            PSZCRO     pszDriverName,
            PSZCRO     pszDeviceName,
            PSZCRO     pszShortName,
            PSZCRO     pszJobProperties,
            PSZCRO     pszXMLDeviceName)
   : PrintDevice (pszDriverName,
                  pszDeviceName,
                  pszShortName,
                  "libUPDFOmniDevice.so",
                  OMNI_CLASS_UPDF,
                  pszJobProperties)
{
   pszDriverName_d = pszDriverName;
   pszDeviceName_d = pszDeviceName;
   pszShortName_d  = pszShortName;

   commonInit (docDC, docUDR);
}

UPDFDevice::
~UPDFDevice ()
{
   if (docDC_d)
   {
      xmlFreeDoc (docDC_d);
      docDC_d = 0;
   }
   if (docUDR_d)
   {
      xmlFreeDoc (docUDR_d);
      docUDR_d = 0;
   }
   if (docLocale_d)
   {
      xmlFreeDoc (docLocale_d);
      docLocale_d = 0;
   }
   if (docCS_d)
   {
      xmlFreeDoc (docCS_d);
      docCS_d = 0;
   }
   if (pszDriverName_d)
   {
      free ((void *)pszDriverName_d);
      pszDriverName_d = 0;
   }
   if (pszDeviceName_d)
   {
      free ((void *)pszDeviceName_d);
      pszDeviceName_d = 0;
   }
   if (pszShortName_d)
   {
      free ((void *)pszShortName_d);
      pszShortName_d = 0;
   }
}

#ifdef INCLUDE_JP_UPDF_BOOKLET
DeviceBooklet * UPDFDevice::
getDefaultBooklet ()
{
   // @TBD
   return 0;
}
#endif

#ifdef INCLUDE_JP_COMMON_COPIES
DeviceCopies * UPDFDevice::
getDefaultCopies ()
{
   DeviceCopies *pDCRet   = 0;
   PSZRO         pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaCopy", "ID");

      if (pszValue)
      {
         pDCRet = UPDFDeviceCopies::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDCRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDCRet)
      return pDCRet;
   else
      // @TBD - hard defaults
      return 0;
}
#endif

PSZCRO UPDFDevice::
getDefaultDitherID ()
{
   // @TBD
   return "DITHER_STUCKI_DIFFUSION";
}

DeviceForm * UPDFDevice::
getDefaultForm ()
{
   DeviceForm *pDFRet   = 0;
   PSZRO       pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaSize", "ID");

      if (pszValue)
      {
         pDFRet = UPDFDeviceForm::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDFRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDFRet)
      return pDFRet;
   else
      // @TBD - hard defaults
      return UPDFDeviceForm::create (this, DeviceForm::FORM_LETTER);
}

#ifdef INCLUDE_JP_UPDF_JOGGING
DeviceJogging * UPDFDevice::
getDefaultJogging ()
{
   DeviceJogging *pDJRet   = 0;
   PSZRO          pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaJogging", "ID");

      if (pszValue)
      {
         pDJRet = UPDFDeviceJogging::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDJRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDJRet)
      return pDJRet;
   else
      // @TBD - hard defaults
      return 0;
}
#endif

DeviceMedia * UPDFDevice::
getDefaultMedia ()
{
   DeviceMedia *pDMRet   = 0;
   PSZRO        pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaType", "ID");

      if (pszValue)
      {
         pDMRet = UPDFDeviceMedia::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDMRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDMRet)
      return pDMRet;
   else
      // @TBD - hard defaults
      return UPDFDeviceMedia::create (this, DeviceMedia::MEDIA_PLAIN);
}

#ifdef INCLUDE_JP_COMMON_NUP
DeviceNUp * UPDFDevice::
getDefaultNUp ()
{
   DeviceNUp *pDNUpRet = 0;
   PSZRO      pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaNUp", "ID");

      if (pszValue)
      {
         pDNUpRet = UPDFDeviceNUp::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDNUpRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDNUpRet)
      return pDNUpRet;
   else
      // @TBD - hard defaults
      return 0;
}
#endif

DeviceOrientation * UPDFDevice::
getDefaultOrientation ()
{
   DeviceOrientation *pDORet         = 0;
   PSZRO              pszOrientation = 0;
   PSZRO              pszRotation    = 0;

   if (pInstance_d)
   {
      pszOrientation = pInstance_d->getXMLObjectValue ("MediaPageOrientation",
                                                       "ID");
      pszRotation    = pInstance_d->getXMLObjectValue ("MediaPageRotation",
                                                       "ID");

      if (  pszOrientation
         && pszRotation
         )
      {
         pDORet = UPDFDeviceOrientation::create (this,
                                                 pszOrientation,
                                                 pszRotation);
      }
   }

#ifndef RETAIL
   if (!pDORet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszOrientation = " << pszOrientation << ", pszRotation = " << pszRotation << std::endl;
   }
#endif

   if (pszOrientation)
   {
      free ((void *)pszOrientation);
   }

   if (pszRotation)
   {
      free ((void *)pszRotation);
   }

   if (pDORet)
      return pDORet;
   else
      // @TBD - hard defaults
      return UPDFDeviceOrientation::create (this, DeviceOrientation::ORIENTATION_PORTRAIT);
}

#ifdef INCLUDE_JP_COMMON_OUTPUT_BIN
DeviceOutputBin * UPDFDevice::
getDefaultOutputBin ()
{
   // @TBD
   return 0;
}
#endif

DevicePrintMode * UPDFDevice::
getDefaultPrintMode ()
{
   // @TBD
   return UPDFDevicePrintMode::create (this, DevicePrintMode::PRINT_MODE_1_ANY);
}

DeviceResolution * UPDFDevice::
getDefaultResolution ()
{
   DeviceResolution *pDRRet   = 0;
   PSZRO             pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("DeviceResolution", "ID");

      if (pszValue)
      {
         pDRRet = UPDFDeviceResolution::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDRRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDRRet)
      return pDRRet;
   else
      // @TBD - hard defaults
      return UPDFDeviceResolution::create (this, DeviceResolution::RESOLUTION_300_X_300);
}

#ifdef INCLUDE_JP_COMMON_SCALING
DeviceScaling * UPDFDevice::
getDefaultScaling ()
{
   // @TBD
   return 0;
}
#endif

#ifdef INCLUDE_JP_COMMON_COLLATE
DeviceSheetCollate * UPDFDevice::
getDefaultSheetCollate ()
{
   // @TBD
   return 0;
}
#endif

#ifdef INCLUDE_JP_COMMON_SIDE
DeviceSide * UPDFDevice::
getDefaultSides ()
{
   DeviceSide  *pDDRet   = 0;
   PSZRO        pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaDuplex", "ID");

      if (pszValue)
      {
         pDDRet = UPDFDeviceSide::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDDRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDDRet)
      return pDDRet;
   else
      // @TBD - hard defaults
      return 0;
}
#endif

#ifdef INCLUDE_JP_COMMON_STITCHING
DeviceStitching * UPDFDevice::
getDefaultStitching ()
{
   // @TBD
   return 0;
}
#endif

DeviceTray * UPDFDevice::
getDefaultTray ()
{
   DeviceTray *pDTRet   = 0;
   PSZRO       pszValue = 0;

   if (pInstance_d)
   {
      pszValue = pInstance_d->getXMLObjectValue ("MediaSource", "ID");

      if (pszValue)
      {
         pDTRet = UPDFDeviceTray::create (this, pszValue);
      }
   }

#ifndef RETAIL
   if (!pDTRet)
   {
      if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice: " << __FUNCTION__ << ": Error: pszValue = " << pszValue << std::endl;
   }
#endif

   if (pszValue)
   {
      free ((void *)pszValue);
   }

   if (pDTRet)
      return pDTRet;
   else
      // @TBD - hard defaults
      return UPDFDeviceTray::create (this, DeviceTray::TRAY_AUTO);
}

#ifdef INCLUDE_JP_COMMON_TRIMMING
DeviceTrimming * UPDFDevice::
getDefaultTrimming ()
{
   // @TBD
   return 0;
}
#endif

DeviceCommand * UPDFDevice::
getDefaultCommands ()
{
   // @TBD
   return 0;
///return new UPDFDeviceCommands (this);
}

DeviceData * UPDFDevice::
getDefaultData ()
{
   // @TBD
   return 0;
}

DeviceString * UPDFDevice::
getDefaultString ()
{
   // @TBD
   return 0;
}

DeviceGamma * UPDFDevice::
getCurrentGamma ()
{
   // @TBD
   return 0;
}

UPDFDevice * UPDFDevice::
isAUPDFDevice (Device *pDevice)
{
   return dynamic_cast<UPDFDevice *>(pDevice);
}

xmlNodePtr UPDFDevice::
findDCEntry (xmlNodePtr  root,
             PSZCRO      pszName,
             bool        fShouldDebugOutput)
{
   if (root == 0)
   {
      root = nodeRootDC_d;
   }

   return findEntry (root, pszName, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findUDREntry (xmlNodePtr  root,
              PSZCRO      pszName,
              bool        fShouldDebugOutput)
{
   if (root == 0)
   {
      root = nodeRootUDR_d;
   }

   return findEntry (root, pszName, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findLocaleEntry (xmlNodePtr  root,
                 PSZCRO      pszName,
                 bool        fShouldDebugOutput)
{
   if (root == 0)
   {
      root = nodeRootLocale_d;
   }

   return findEntry (root, pszName, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findCSEntry (xmlNodePtr  root,
             PSZCRO      pszName,
             bool        fShouldDebugOutput)
{
   if (root == 0)
   {
      root = nodeRootCS_d;
   }

   return findEntry (root, pszName, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findDCEntryKeyValue (PSZCRO pszKey,
                     PSZCRO pszValue,
                     bool   fShouldDebugOutput)
{
   return findEntryKeyValue (nodeRootDC_d, pszKey, pszValue, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findUDREntryKeyValue (PSZCRO pszKey,
                      PSZCRO pszValue,
                      bool   fShouldDebugOutput)
{
   return findEntryKeyValue (nodeRootUDR_d, pszKey, pszValue, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findLocaleEntryKeyValue (PSZCRO pszKey,
                         PSZCRO pszValue,
                         bool   fShouldDebugOutput)
{
   return findEntryKeyValue (nodeRootLocale_d, pszKey, pszValue, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findCSEntryKeyValue (PSZCRO pszKey,
                     PSZCRO pszValue,
                     bool   fShouldDebugOutput)
{
   return findEntryKeyValue (nodeRootCS_d, pszKey, pszValue, fShouldDebugOutput);
}

xmlNodePtr UPDFDevice::
findEntry (xmlNodePtr  root,
           PSZCRO      pszName,
           bool        fShouldDebugOutput)
{
   if (root == 0)
      return 0;

   xmlNodePtr elm = firstNode (root->xmlChildrenNode);

   while (elm != 0)
   {
#ifndef RETAIL
//////if (DebugOutput::shouldOutputUPDFDevice ()) DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": " << elm->name << " == " << pszName << std::endl;
#endif

      if (0 == strcmp ((PSZCRO)elm->name, pszName))
      {
         break;
      }

      elm = nextNode (elm);
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDevice ())
   {
      if (!elm)
      {
         DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": Did not find " << pszName << std::endl;
      }
   }
#endif

   return elm;
}

xmlNodePtr UPDFDevice::
findEntryKeyValue (xmlNodePtr  root,
                   PSZCRO      pszKey,
                   PSZCRO      pszValue,
                   bool        fShouldDebugOutput)
{
   if (root == 0)
      return 0;

   xmlNodePtr elm      = root;
   xmlNodePtr elmFound = 0;

   while (elm != 0)
   {
      PSZCRO pszElmValue = (PSZCRO)xmlGetProp (elm, (xmlChar *)pszKey);

      if (  pszElmValue
         && 0 == strcmp (pszElmValue, pszValue)
         )
      {
         elmFound = elm;
      }

      if (pszElmValue)
      {
         free ((void *)pszElmValue);
      }

      if (  !elmFound
         && elm->xmlChildrenNode
         )
      {
         elmFound = findEntryKeyValue (elm->xmlChildrenNode, pszKey, pszValue, false);
      }

      if (elmFound)
      {
         elm = elmFound;
         break;
      }

      elm = nextNode (elm);
   }

#ifndef RETAIL
   if (fShouldDebugOutput)
   {
      if (elm)
      {
         DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": Found ";
      }
      else
      {
         DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": Did not find ";
      }
      DebugOutput::getErrorStream () << pszValue << std::endl;
   }
#endif

   return elm;
}

bool UPDFDevice::
setVirtualUnits ()
{
   xmlNodePtr nodeHeader = 0;
   int        iRet       = 0;

   if (  ((nodeHeader = findUDREntry (nodeHeader, "Device_Features",   DebugOutput::shouldOutputUPDFDevice ())) != 0)
      && ((nodeHeader = findUDREntry (nodeHeader, "PrintCapabilities", DebugOutput::shouldOutputUPDFDevice ())) != 0)
      && ((nodeHeader = findUDREntry (nodeHeader, "Header",            DebugOutput::shouldOutputUPDFDevice ())) != 0)
      && ((nodeHeader = findUDREntry (nodeHeader, "VirtualUnits",      DebugOutput::shouldOutputUPDFDevice ())) != 0)
      )
   {
      PSZCRO pszVirtualUnits = (PSZCRO)xmlGetProp (nodeHeader,
                                                   (xmlChar *)"Predefined_ID");
      if (pszVirtualUnits)
      {
         int iX   = 0,
             iY   = 0;

         iRet = sscanf (pszVirtualUnits, "virtualUnits_%dx%d", &iX, &iY);

#ifndef RETAIL
         if (DebugOutput::shouldOutputUPDFDevice ())
            DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": iRet = " << iRet << ", iX = " << iX << ", iY = " << iY << std::endl;
#endif

         if (2 == iRet)
         {
            iVirtualUnitsX_d = iX;
            iVirtualUnitsY_d = iY;
         }

         free ((void *)pszVirtualUnits);
      }
   }

#ifndef RETAIL
   if (  !iVirtualUnitsX_d
      || !iVirtualUnitsY_d
      )
   {
      DebugOutput::getErrorStream () << "UPDFDevice::" << __FUNCTION__ << ": Error: Could not find virtual units. nodeHeader = " << std::hex << (int)nodeHeader << std::dec << std::endl;
   }
#endif

   return iRet == true;
}

int UPDFDevice::
getXVirtualUnits ()
{
   return iVirtualUnitsX_d;
}

int UPDFDevice::
getYVirtualUnits ()
{
   return iVirtualUnitsY_d;
}

xmlNodePtr UPDFDevice::
getRootDC ()
{
   return nodeRootDC_d;
}

xmlNodePtr UPDFDevice::
getRootUDR ()
{
   return nodeRootUDR_d;
}

xmlNodePtr UPDFDevice::
getRootLocale ()
{
   return nodeRootLocale_d;
}

xmlNodePtr UPDFDevice::
getRootCS ()
{
   return nodeRootCS_d;
}
