/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <UPDFDeviceCopies.hpp>

UPDFDeviceCopies::
UPDFDeviceCopies (Device      *pDevice,
                  PSZRO        pszJobProperties,
                  BinaryData  *data,
                  xmlNodePtr   node,
                  int          iMinimum,
                  int          iMaximum,
                  bool         fSimulationRequired)
   : DeviceCopies (pDevice,
                   pszJobProperties,
                   data,
                   iMinimum,
                   iMaximum,
                   fSimulationRequired)
{
   node_d = node;
}

UPDFDeviceCopies::
~UPDFDeviceCopies ()
{
}

DeviceCopies * UPDFDeviceCopies::
create (Device *pDevice,
        PSZCRO  pszJobProperties)
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice);
   xmlNodePtr  nodeCopies  = 0;
   int         iCopies     = -1;

   if (!getComponents (pszJobProperties,
                       &iCopies))
   {
      return 0;
   }

   nodeCopies = findCopies (pUPDFDevice);

   if (!nodeCopies)
   {
      return 0;
   }

   PSZCRO        pszMaximum = (PSZCRO)xmlGetProp (nodeCopies, (xmlChar *)"MaximumValue");
   int           iMaximum   = -1;
   DeviceCopies *pRet       = 0;

   if (pszMaximum)
   {
      sscanf (pszMaximum, "%d", &iMaximum);
   }

   if (  1       <= iCopies
      && iCopies <= iMaximum)
   {
      BinaryData *data                = 0;    // @TBD
      bool        fSimulationRequired = true; // @TBD

      pRet = new UPDFDeviceCopies (pDevice,
                                   pszJobProperties,
                                   data,
                                   nodeCopies,
                                   1,
                                   iMaximum,
                                   fSimulationRequired);
   }

   if (pszMaximum)
   {
      free ((void *)pszMaximum);
   }

   return pRet;
}

PSZCRO UPDFDeviceCopies::
getDeviceID ()
{
   return 0;
}

bool UPDFDeviceCopies::
isSupported (PSZCRO pszJobProperties)
{
   return false; // @TBD
}

xmlNodePtr UPDFDeviceCopies::
findCopies (UPDFDevice *pUPDFDevice)
{
   xmlNodePtr nodeCopies = 0;

   if (!pUPDFDevice)
   {
      return nodeCopies;
   }

   if (  ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "Device_Features",    UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "PrintCapabilities",  UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "Features",           UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "PrintMediaHandling", UPDFDeviceCopies)) != 0)
      && ((nodeCopies = FINDUDRENTRY (pUPDFDevice, nodeCopies, "MediaCopy",          UPDFDeviceCopies)) != 0)
      )
      return nodeCopies;

   return nodeCopies;
}

std::string UPDFDeviceCopies::
toString (std::ostringstream& oss)
{
   std::ostringstream oss2;

   oss << "{"
       << DeviceCopies::toString (oss2)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
std::ostream&
operator<< (std::ostream& os, const UPDFDeviceCopies& const_self)
{
   UPDFDeviceCopies&  self = const_cast<UPDFDeviceCopies&>(const_self);
   std::ostringstream oss;

   os << self.toString (oss);

   return os;
}
