/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <Device.hpp>

Device *
createDevice (char     *pszDriverLibrary,
              char     *pszJobProperties,
              bool      fAdvanced,
              GModule **phmodDevice)
{
#define LIBDEVICENAME3(s) STRINGIZE(s)
#define LIBDEVICENAME2(x) LIBDEVICENAME3(lib##x##.so)
#define LIBDEVICENAME1(x) LIBDEVICENAME2(x)
#define LIBDEVICENAME     LIBDEVICENAME1(DEVICENAME)

   char               achLibDeviceName[512]; // @TBD - need max path define
   Device            *pDevice           = 0;
   PFNNEWDEVICE       pfnNewDevice      = 0;
   PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;

   if (  0 == pszDriverLibrary
      || !*pszDriverLibrary
      )
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pszDriverLibrary is not filled in!" << std::endl;
      return 0;
   }

   if (0 != strncasecmp (pszDriverLibrary, "lib", 3))
   {
      sprintf (achLibDeviceName, "lib%s.so", pszDriverLibrary);
   }
   else
   {
      strcpy (achLibDeviceName, pszDriverLibrary);
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": Trying to load " << pszDriverLibrary << std::endl;
   }
#endif

   *phmodDevice = g_module_open (pszDriverLibrary, (GModuleFlags)0);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": *phmodDevice = " << std::hex << *phmodDevice << std::dec << std::endl;
   }
#endif

   if (!*phmodDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << std::endl;
      return 0;
   }

   // nm libEpson_Stylus_Color_860.so | grep newDevice

   // returns 00011e20 T newDeviceW_Advanced for
   // extern Device *newDevice (bool fAdvanced);
   g_module_symbol (*phmodDevice, "newDeviceW_Advanced", (gpointer *)&pfnNewDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDevice = 0x" << std::hex << (int)pfnNewDevice << std::dec << std::endl;
   }
#endif

   if (!pfnNewDevice)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << std::endl;

      return 0;
   }

   // returns 00011d30 T newDeviceW_JopProp_Advanced for
   // extern Device *newDevice (char *pszJobProperties, bool fAdvanced);
   g_module_symbol (*phmodDevice, "newDeviceW_JopProp_Advanced", (gpointer *)&pfnNewDeviceWArgs);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDeviceWArgs = 0x" << std::hex << (int)pfnNewDeviceWArgs << std::dec << std::endl;
   }
#endif

   if (!pfnNewDeviceWArgs)
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << std::endl;

      return 0;
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": ********** Creating device with passed in properties **********" << std::endl;
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": The Properties are: " << pszJobProperties << std::endl;
   }
#endif

   pDevice = pfnNewDeviceWArgs (pszJobProperties, fAdvanced);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester ())
   {
      DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": pfnNewDeviceWArgs (...) returns " << *pDevice << std::endl;
   }
#endif

   return pDevice;
}

void
printUsage (char *pszExeName)
{
   DebugOutput::getErrorStream () << "Usage: " << pszExeName << " required_parms optional_parms" << std::endl
      << "   where required_parms are:" << std::endl
      << "      UPDFMasterFile device_name" << std::endl
      << "   where optional_parms are:" << std::endl
      << "      orientation orientation_id" << std::endl
      << "      form form_id" << std::endl
      << "      tray tray_id" << std::endl
      << "      media media_id" << std::endl
      << "      resolution resolution_id" << std::endl
      << "      dithering dithering_id" << std::endl
      << "      printmode printmode_id" << std::endl
      << "      OtherJobPropertyKey value" << std::endl;
}

int
main (int argc, char *argv[])
{
#define JP_BLOCK_SIZE  256
   char              *pszDriverLibrary  = "libUPDFDevice.so";
   std::string        stringJobProp     = "";
   GModule           *hmodDevice        = 0;
   Enumeration       *pEnum             = 0;
   Device            *pDevice           = 0;
   DeviceOrientation *pDO               = 0;
   DeviceForm        *pDF               = 0;
   HardCopyCap       *pHCC              = 0;
   DeviceTray        *pDT               = 0;
   DeviceMedia       *pDM               = 0;
   DeviceResolution  *pDR               = 0;
   DevicePrintMode   *pDPM              = 0;
#ifdef INCLUDE_JP_COMMON_SIDE
   DeviceSide        *pDS               = 0;
#endif
#ifdef INCLUDE_JP_COMMON_NUP
   DeviceNUp         *pDNUp             = 0;
#endif
#ifdef INCLUDE_JP_COMMON_COPIES
   DeviceCopies      *pDC               = 0;
#endif
#ifdef INCLUDE_JP_UPDF_JOGGING
   DeviceJogging     *pDJ               = 0;
#endif
   int                rc                = 0;

   for (int i = 1; i < argc; i++)
   {
      // Assume that it is a Job Property (where there needs to be at least
      // 2 parameters)
      if (2 <= argc - i)
      {
         bool fSpaceFound = false;

         if (strchr (argv[i + 1], ' '))
         {
            fSpaceFound = true;
         }

         if (stringJobProp.c_str ()[0])
            stringJobProp += " ";
         stringJobProp += argv[i];
         stringJobProp += "=";
         if (fSpaceFound)
            stringJobProp += "\"";
         stringJobProp += argv[i + 1];
         if (fSpaceFound)
            stringJobProp += "\"";

         i++;
      }
      else
      {
         printUsage (argv[0]);

         rc = 4;
         goto BUGOUT;
      }
   }

   std::cerr << "The job properties are \"" << stringJobProp << "\"" << std::endl;

   pDevice = createDevice (pszDriverLibrary,
                           (char *)stringJobProp.c_str (),
                           true,
                           &hmodDevice);

   pDO = pDevice->getCurrentOrientation ();
   pEnum = pDO->getEnumeration ();

   std::cout << "Current orientation is:" << std::endl;
   std::cout << DeviceOrientation::IDToName (pDO->getID ()) << std::endl;

   std::cout << "Portrait is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_PORTRAIT))
      std::cout << " not";
   std::cout << " supported." << std::endl;
   std::cout << "Landscape is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_LANDSCAPE))
      std::cout << " not";
   std::cout << " supported." << std::endl;
   std::cout << "Reverse-Portrait is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_REVERSE_PORTRAIT))
      std::cout << " not";
   std::cout << " supported." << std::endl;
   std::cout << "Reverse-Landscape is";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_REVERSE_LANDSCAPE))
      std::cout << " not";
   std::cout << " supported." << std::endl;

   std::cout << "Enumerating orientations:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDO = (DeviceOrientation *)pEnum->nextElement ();

      std::cout << "   " << DeviceOrientation::IDToName (pDO->getID ());

      if (pDO->needsSimulation ())
         std::cout << " [software simulation required]";

      std::cout << std::endl;

      delete pDO;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

   pDF = pDevice->getCurrentForm ();
   pEnum = pDF->getEnumeration ();

   std::cout << "Current form is:" << std::endl;
   std::cout << DeviceForm::IDToName (pDF->getID ()) << std::endl;
   pHCC = pDF->getHardCopyCap ();
   std::cout << "Form dimensions are: ("
             << pHCC->getCx ()
             << ","
             << pHCC->getCy ()
             << ") (L:"
             << pHCC->getLeftClip ()
             << ",T:"
             << pHCC->getTopClip ()
             << ",R:"
             << pHCC->getRightClip ()
             << ",B:"
             << pHCC->getBottomClip ()
             << ")"
             << std::endl;

   std::cout << "Enumerating forms:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      std::cout << "   " << DeviceForm::IDToName (pDF->getID ()) << std::endl;

      delete pDF;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

   pDT = pDevice->getCurrentTray ();
   pEnum = pDT->getEnumeration ();

   std::cout << "Current tray is:" << std::endl;
   std::cout << DeviceTray::IDToName (pDT->getID ()) << std::endl;

   std::cout << "Enumerating trays:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDT = (DeviceTray *)pEnum->nextElement ();

      std::cout << "   " << DeviceTray::IDToName (pDT->getID ()) << std::endl;

      delete pDT;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

   pDM = pDevice->getCurrentMedia ();
   pEnum = pDM->getEnumeration ();

   std::cout << "Current media is:" << std::endl;
   std::cout << DeviceMedia::IDToName (pDM->getID ()) << std::endl;

   std::cout << "Enumerating medias:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDM = (DeviceMedia *)pEnum->nextElement ();

      std::cout << "   " << DeviceMedia::IDToName (pDM->getID ()) << std::endl;

      delete pDM;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

   pDR = pDevice->getCurrentResolution ();
   pEnum = pDR->getEnumeration ();

   std::cout << "Current resolution is:" << std::endl;
   std::cout << DeviceResolution::IDToName (pDR->getID ()) << std::endl;

   std::cout << "Resolution dimensions are: ("
             << pDR->getXRes ()
             << ","
             << pDR->getYRes ()
             << ")"
             << std::endl;

   std::cout << "Enumerating resolutions:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDR = (DeviceResolution *)pEnum->nextElement ();

      std::cout << "   " << DeviceResolution::IDToName (pDR->getID ()) << std::endl;

      delete pDR;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

   pDPM = pDevice->getCurrentPrintMode ();
   pEnum = pDPM->getEnumeration ();

   std::cout << "Current print mode is:" << std::endl;
   std::cout << DevicePrintMode::IDToName (pDPM->getID ()) << std::endl;

   std::cout << "Print Mode dimensions are: (CT:"
             << pDPM->getColorTech ()
             << ",P:"
             << pDPM->getPhysicalCount ()
             << ",L:"
             << pDPM->getLogicalCount ()
             << ",#planes:"
             << pDPM->getNumPlanes ()
             << ")"
             << std::endl;

   std::cout << "Enumerating print modes:" << std::endl;
   while (pEnum->hasMoreElements ())
   {
      pDPM = (DevicePrintMode *)pEnum->nextElement ();

      std::cout << "   " << DevicePrintMode::IDToName (pDPM->getID ()) << std::endl;

      delete pDPM;
   }
   std::cout << "Done!" << std::endl;

   delete pEnum;

#ifdef INCLUDE_JP_COMMON_SIDE
   std::cout << "Current duplex mode is:" << std::endl;
   pDS = pDevice->getCurrentSides ();
   if (pDS)
   {
      pEnum = pDS->getEnumeration ();

      std::cout << DeviceSide::IDToName (pDS->getID ()) << std::endl;

      std::cout << "Enumerating duplexes:" << std::endl;
      while (pEnum->hasMoreElements ())
      {
         pDS = (DeviceSide *)pEnum->nextElement ();

         std::cout << "   " << DeviceSide::IDToName (pDS->getID ()) << std::endl;

         delete pDS;
      }
      std::cout << "Done!" << std::endl;

      delete pEnum;
   }
   else
   {
      std::cout << "Not defined!" << std::endl;
   }
#endif

#ifdef INCLUDE_JP_COMMON_NUP
   std::cout << "Current nup mode is:" << std::endl;
   pDNUp = pDevice->getCurrentNUp ();
   if (pDNUp)
   {
      pEnum = pDNUp->getEnumeration ();

      std::cout << DeviceNUp::IDToName (pDNUp->getID ()) << std::endl;

      std::cout << "Enumerating nups:" << std::endl;
      while (pEnum->hasMoreElements ())
      {
         pDNUp = (DeviceNUp *)pEnum->nextElement ();

         std::cout << "   " << DeviceNUp::IDToName (pDNUp->getID ());

         if (pDNUp->getHorizontalPages () != pDNUp->getVerticalPages ())
         {
            std::cout << " ("
                      << pDNUp->getHorizontalPages ()
                      << ", "
                      << pDNUp->getVerticalPages ()
                      << ")";
         }

         std::cout << std::endl;

         delete pDNUp;
      }
      std::cout << "Done!" << std::endl;

      delete pEnum;
   }
   else
   {
      std::cout << "Not defined!" << std::endl;
   }
#endif

#ifdef INCLUDE_JP_COMMON_COPIES
   std::cout << "Current copies is:" << std::endl;
   pDC = pDevice->getCurrentCopies ();
   if (pDC)
   {
      std::cout << pDC->getNumCopies () << std::endl;

      std::cout << "The minimum # copies is " << pDC->getMinimum () << std::endl;
      std::cout << "The maximum # copies is " << pDC->getMaximum () << std::endl;
   }
   else
   {
      std::cout << "Not defined!" << std::endl;
   }
#endif

#ifdef INCLUDE_JP_UPDF_JOGGING
   std::cout << "Current jogging mode is:" << std::endl;
   pDJ = pDevice->getCurrentJogging ();
   if (pDJ)
   {
      pEnum = pDJ->getEnumeration ();

      std::cout << DeviceJogging::IDToName (pDJ->getID ()) << std::endl;

      std::cout << "Enumerating joggings:" << std::endl;
      while (pEnum->hasMoreElements ())
      {
         pDJ = (DeviceJogging *)pEnum->nextElement ();

         std::cout << "   " << DeviceJogging::IDToName (pDJ->getID ()) << std::endl;

         delete pDJ;
      }
      std::cout << "Done!" << std::endl;

      delete pEnum;
   }
   else
   {
      std::cout << "Not defined!" << std::endl;
   }
#endif

   rc = 0;

BUGOUT:
   // Clean up
   delete pDevice;

   if (hmodDevice)
   {
#ifndef RETAIL
      int rc2 =
#endif
           g_module_close (hmodDevice);

#ifndef RETAIL
      if (DebugOutput::shouldOutputDeviceTester ())
      {
         DebugOutput::getErrorStream () << "DeviceTester::" << __FUNCTION__ << ": g_module_close returns " << rc2 << std::endl;
      }
#endif
   }

   return rc;
}
