/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <UPDFDeviceTray.hpp>

/*
*/
static int     getOmniTrayID             (xmlNodePtr      node);

UPDFDeviceTray::
UPDFDeviceTray (Device      *pDevice,
                int          id,
                int          iType,
                BinaryData  *data,
                xmlNodePtr   node)
   : DeviceTray (pDevice,
                 id,
                 iType,
                 data)
{
   node_d = node;
}

UPDFDeviceTray::
~UPDFDeviceTray ()
{
}

DeviceTray * UPDFDeviceTray::
createV (int id)
{
   return UPDFDeviceTray::create (pDevice_d, id);
}

DeviceTray * UPDFDeviceTray::
create (Device *pDevice,
        PSZCRO  pszId)
{
   int id = DeviceTray::nameToID (pszId);

   if (DeviceTray::TRAY_UNLISTED != id)
   {
      return create (pDevice, id);
   }
   else
   {
      UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice);
      xmlNodePtr   nodeTrays   = 0;

      nodeTrays = findTrays (pUPDFDevice);

      if (nodeTrays)
      {
         xmlNodePtr nodeItem  = firstNode (nodeTrays->xmlChildrenNode);
         xmlNodePtr nodeFound = 0;

         while (nodeItem != 0)
         {
            PSZCRO pszNodeId = (PSZCRO)xmlGetProp (nodeItem, (xmlChar *)"ID");

            if (pszNodeId)
            {
               if (0 == strcmp (pszNodeId, pszId))
               {
                  nodeFound = nodeItem;
               }

               free ((void *)pszNodeId);
            }

            if (nodeFound)
            {
               id = getOmniTrayID (nodeFound);

               if (DeviceTray::TRAY_UNLISTED != id)
               {
                  return create (pDevice, id);
               }

               nodeFound = 0;
            }

            nodeItem = nextNode (nodeItem);
         }
      }

      return 0;
   }
}

static int
getOmniTrayID (xmlNodePtr node)
{
   int id = DeviceTray::TRAY_UNLISTED;

   if (!node)
   {
      return id;
   }

   PSZCRO pszID = (PSZCRO)xmlGetProp (node, (xmlChar *)"ID");

   if (pszID)
   {
      if (0 == strcasecmp (pszID, "Source_Auto"))
         id = DeviceTray::TRAY_AUTO;
      else if (0 == strcasecmp (pszID, "Source_Tray2"))
         id = DeviceTray::TRAY_TRAY_2;
      else if (0 == strcasecmp (pszID, "Source_Tray3"))
         id = DeviceTray::TRAY_TRAY_3;

      free ((void *)pszID);
   }

   return id;
}

static xmlNodePtr
skipInvalidTrays (xmlNodePtr node)
{
   if (!node)
   {
      return 0;
   }

   int id = getOmniTrayID (node);

   while (  node
         && (  id == DeviceTray::TRAY_UNLISTED
            || id == DeviceTray::TRAY_NONE
            )
         )
   {
      node = nextNode (node);
      id   = getOmniTrayID (node);
   }

   return node;
}

static DeviceTray *
createFromXMLNode (Device      *pDevice,
                   xmlNodePtr   node,
                   int          id)
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice);

   if (!pUPDFDevice)
   {
      return 0;
   }

   int          iType = 0; // @TBD
   BinaryData  *data  = 0; // @TBD

   return new UPDFDeviceTray (pUPDFDevice,
                              id,
                              iType,
                              data,
                              node);
}

DeviceTray * UPDFDeviceTray::
create (Device *pDevice, int id)
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputUPDFDeviceTray ()) DebugOutput::getErrorStream () << "UPDFDeviceTray::" << __FUNCTION__ << ": pDevice = " << pDevice << ", id = " << id << std::endl;
#endif

   if (!pUPDFDevice)
   {
      return 0;
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      while (nodeItem != 0)
      {
         int iTrayID = getOmniTrayID (nodeItem);

         if (  id == iTrayID
            && !(  id == DeviceTray::TRAY_UNLISTED
                || id == DeviceTray::TRAY_NONE
                )
            )
         {
            return createFromXMLNode (pDevice, nodeItem, id);
         }

         nodeItem = nextNode (nodeItem);
      }

      return 0;
   }
   else
   {
      return 0;
   }
}

PSZCRO UPDFDeviceTray::
getDeviceID ()
{
   return 0;
}

bool UPDFDeviceTray::
isTraySupported (int id)
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice_d);

   if (!pUPDFDevice)
   {
      return 0;
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      while (nodeItem != 0)
      {
         int iTrayID = getOmniTrayID (nodeItem);

         if (  id == iTrayID
            && !(  id == DeviceTray::TRAY_UNLISTED
                || id == DeviceTray::TRAY_NONE
                )
            )
         {
            return true;
         }

         nodeItem = nextNode (nodeItem);
      }
   }

   return false;
}

class UPDFTray_Enumerator : public Enumeration
{
public:
   UPDFTray_Enumerator (Device *pDevice, xmlNodePtr nodeItem)
   {
      pDevice_d  = pDevice;
      nodeItem_d = skipInvalidTrays (nodeItem);
   }

   virtual bool hasMoreElements ()
   {
      return nodeItem_d ? true : false;
   }

   virtual void *nextElement ()
   {
      void *pvRet = 0;

      if (nodeItem_d)
      {
         int id = getOmniTrayID (nodeItem_d);

         pvRet = createFromXMLNode (pDevice_d,
                                    nodeItem_d,
                                    id);

         nodeItem_d = nextNode (nodeItem_d);
         nodeItem_d = skipInvalidTrays (nodeItem_d);
      }

      return pvRet;
   }

private:
   Device     *pDevice_d;
   xmlNodePtr  nodeItem_d;
};

Enumeration * UPDFDeviceTray::
getEnumeration ()
{
   UPDFDevice *pUPDFDevice = UPDFDevice::isAUPDFDevice (pDevice_d);

   if (!pUPDFDevice)
   {
      return new UPDFTray_Enumerator (pDevice_d, 0);
   }

   xmlNodePtr nodeTrays = 0;

   nodeTrays = findTrays (pUPDFDevice);

   if (nodeTrays)
   {
      xmlNodePtr nodeItem = firstNode (nodeTrays->xmlChildrenNode);

      return new UPDFTray_Enumerator (pDevice_d, nodeItem);
   }
   else
   {
      return new UPDFTray_Enumerator (pDevice_d, 0);
   }
}

xmlNodePtr UPDFDeviceTray::
findTrays (UPDFDevice *pUPDFDevice)
{
   xmlNodePtr nodeTrays = 0;

   if (!pUPDFDevice)
   {
      return nodeTrays;
   }

   if (  ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "Device_Features",    UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "PrintCapabilities",  UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "Features",           UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "PrintMediaHandling", UPDFDeviceTray)) != 0)
      && ((nodeTrays = FINDUDRENTRY (pUPDFDevice, nodeTrays, "MediaSource",        UPDFDeviceTray)) != 0)
      )
      return nodeTrays;

   return nodeTrays;
}

std::string UPDFDeviceTray::
toString (std::ostringstream& oss)
{
   std::ostringstream oss2;

   oss << "{"
       << DeviceTray::toString (oss2)
       << "}";

   return oss.str ();
}

/* Provide a way to print out class data
*/
std::ostream&
operator<< (std::ostream& os, const UPDFDeviceTray& const_self)
{
   UPDFDeviceTray&    self = const_cast<UPDFDeviceTray&>(const_self);
   std::ostringstream oss;

   os << self.toString (oss);

   return os;
}
