/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _OmniDomParser
#define _OmniDomParser

#include <string>
#include <fstream>

#include <libxml/parser.h>
#include <libxml/tree.h>
//#define DTD 1
#define SCHEMA 1
#ifdef SCHEMA
#include <libxml/xmlschemas.h>
#include <libxml/xmlschemastypes.h>
#endif

#include "Device.hpp"
#include "DeviceInfo.hpp"

class OmniDomParser
{
public:
                 OmniDomParser              (const char  *pszXMLFile,
                                             DeviceInfo  *pdi,
                                             bool         fAutoconf,
                                             bool         fAutoconfNoInst);
   virtual      ~OmniDomParser              ();

   bool          parse                      ();
   void          setErrorCondition          (); // for ReportError interface

   bool          fileModified               ();

private:
   std::string  *getNameOut                 ();
   bool          shouldCreateFile           (std::string       strFileName);
   void          copyFile                   (const char       *pszFileNameIn);
   bool          openOutputFiles            (std::string      *pstringName,
                                             std::ofstream   **ppofstreamHPP,
                                             std::ofstream   **ppofstreamCPP);
   void          addToLibraries3            (const char       *pszFileName);
   void          addToLibraries3            (std::string&      stringFileName);
   bool          parseBinaryData            (const char       *pszData,
                                             byte            **ppbData,
                                             int              *pcbData);
   void          processDOMNode             (xmlNodePtr        node);
   bool          processDeviceCopies        (xmlNodePtr        deviceCopiesNode);
   bool          processDeviceForms         (xmlNodePtr        deviceFormsNode);
   bool          processDeviceMedias        (xmlNodePtr        deviceMediasNode);
   bool          processDeviceNUps          (xmlNodePtr        deviceNUpsNode);
   bool          processDeviceOrientations  (xmlNodePtr        deviceOrientationsNode);
   bool          processDeviceOutputBins    (xmlNodePtr        deviceOutputBinsNode);
   bool          processDevicePrintModes    (xmlNodePtr        devicePrintModesNode);
   bool          processDeviceResolutions   (xmlNodePtr        deviceResolutionsNode);
   bool          processDeviceScalings      (xmlNodePtr        deviceScalingsNode);
   bool          processDeviceSheetCollates (xmlNodePtr        deviceSheetCollatesNode);
   bool          processDeviceSides         (xmlNodePtr        deviceSidesNode);
   bool          processDeviceStitchings    (xmlNodePtr        deviceStitchingsNode);
   bool          processDeviceTrays         (xmlNodePtr        deviceTraysNode);
   bool          processDeviceTrimmings     (xmlNodePtr        deviceTrimmingsNode);
   bool          processDeviceConnections   (xmlNodePtr        deviceConnectionsNode);
   bool          processDeviceGammas        (xmlNodePtr        deviceGammasNode);
   bool          processDeviceCommands      (xmlNodePtr        deviceCommandsNode);
   bool          processDeviceDatas         (xmlNodePtr        deviceDatas);
   bool          processDeviceStrings       (xmlNodePtr        deviceDatas);
   bool          preprocessDriver           (xmlNodePtr        driverNode);
   bool          processDriver              (xmlNodePtr        driverNode);

   xmlDocPtr    doc_d;
   std::string *pstrXMLFile_d;
   DeviceInfo  *pdi_d;
   bool         fErrorCondition_d;
   bool         fAutoconf_d;
   bool         fAutoconfNoInst_d;
   bool         fFileModified_d;
};

#endif
