/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <fstream.h>
#include <string>

#include <sax/AttributeList.hpp>

#include "DebugParser.hpp"
#include "DebugDocumentHandler.hpp"

DebugDocumentHandler::
DebugDocumentHandler ()
{
   stringCharacters_d = 0;
}

DebugDocumentHandler::
~DebugDocumentHandler ()
{
   delete stringCharacters_d;
}

void DebugDocumentHandler::
startDocument ()
{
   cerr << "startDocument ()" << endl;
}

void DebugDocumentHandler::
endDocument ()
{
   cerr << "endDocument ()" << endl;
}

void DebugDocumentHandler::
startElement (const XMLCh *const name, AttributeList &attributes)
{
   cerr << "startElement (" << XMLString::transcode (name) << ")" << endl;

   for (int i = 0; i < (int)attributes.getLength (); i++)
   {
      string name  = XMLString::transcode (attributes.getName (i));
      string type  = XMLString::transcode (attributes.getType (i));
      string value = XMLString::transcode (attributes.getValue (i));

      cerr << name << " = " << value << " (" << type << ")" << endl;
   }
}

void DebugDocumentHandler::
characters (const XMLCh* const chars, const unsigned int length)
{
   delete stringCharacters_d;
   stringCharacters_d = new string (XMLString::transcode (chars));

   cerr << "characters (" << *stringCharacters_d << ", " << length << ")" << endl;
}

void DebugDocumentHandler::
endElement (const XMLCh *const name)
{
   cerr << "endElement (" << XMLString::transcode (name) << ")" << endl;

   if (  0 == strcmp ("Uses", XMLString::transcode (name))
      || 0 == strcmp ("Has", XMLString::transcode (name))
      )
   {
      const char *pszFileName = stringCharacters_d->c_str ();

      DebugParser parser (pszFileName);

      parser.parse ();

      cerr << "test = " << *stringCharacters_d << endl;
   }
}
