/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _OmniDomParser
#define _OmniDomParser

#include <string>
#include <fstream>

#include <util/PlatformUtils.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM_Node.hpp>
#define DTD 1
#ifdef DTD
#include <@TBD>
#endif

#include "MyErrorHandler.hpp"

#include "Device.hpp"
#include "DeviceInfo.hpp"

class OmniDomParser : public ReportError
{
public:
                 OmniDomParser             (const char       *pszXMLFile,
                                            DeviceInfo       *pdi,
                                            bool              fAutoconf,
                                            bool              fAutoconfNoInst);
   virtual      ~OmniDomParser             ();

   bool          parse                     ();
   void          setErrorCondition         (); // for ReportError interface

   bool          fileModified              ();

private:
   std::string  *getNameOut                ();
   bool          shouldCreateFile          (std::string       strFileName);
   void          copyFile                  (const char       *pszFileNameIn);
   bool          openOutputFiles           (std::string      *pstringName,
                                            std::ofstream   **ppofstreamHPP,
                                            std::ofstream   **ppofstreamCPP);
   void          addToLibraries3           (const char       *pszFileName);
   void          addToLibraries3           (std::string&      stringFileName);
   bool          parseBinaryData           (const char       *pszData,
                                            byte            **ppbData,
                                            int              *pcbData);
   void          processDOMNode            (DOM_Node&         node);
   bool          processDeviceOrientations (DOM_Node          deviceOrientationsNode);
   bool          processDeviceCommands     (DOM_Node          deviceCommandNode);
   bool          processDeviceResolutions  (DOM_Node          deviceResolutionNode);
   bool          processDevicePrintModes   (DOM_Node          devicePrintModes);
   bool          processDeviceTrays        (DOM_Node          deviceTrays);
   bool          processDeviceForms        (DOM_Node          deviceForms);
   bool          processDeviceMedias       (DOM_Node          deviceMedias);
   bool          processDeviceConnections  (DOM_Node          deviceConnections);
   bool          processDeviceGammas       (DOM_Node          deviceGammas);
   bool          processDeviceDatas        (DOM_Node          deviceDatas);
   bool          processDeviceStrings      (DOM_Node          deviceStrings);
   bool          preprocessDriver          (DOM_Node          driverNode);
   bool          processDriver             (DOM_Node          driverNode);

   DOMParser   *parser_d;
   std::string *pstrXMLFile_d;
   DeviceInfo  *pdi_d;
   bool         fErrorCondition_d;
   bool         fAutoconf_d;
   bool         fAutoconfNoInst_d;
   bool         fFileModified_d;
};

#endif
