/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <fstream.h>

#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <sax/HandlerBase.hpp>
#include <dom/DOM_Node.hpp>
#include <dom/DOM_NamedNodeMap.hpp>
#include <parsers/DOMParser.hpp>

#include "DebugDocumentHandler.hpp"
#include "MyErrorHandler.hpp"
#include "DebugParser.hpp"

DebugParser::
DebugParser (const char *pszXMLFile)
{
   parser_d = new SAXParser ();

   parser_d->setDoValidation (true);
   parser_d->setDoNamespaces (true);
   parser_d->setDocumentHandler (new DebugDocumentHandler ());
   parser_d->setErrorHandler (new MyErrorHandler (this));

   strXMLFile_d = new string (pszXMLFile);

   fErrorCondition_d = false;
}

DebugParser::
~DebugParser ()
{
   delete parser_d;
   delete strXMLFile_d;
}

void DebugParser::
setErrorCondition ()
{
   fErrorCondition_d = true;
}

void DebugParser::
parse ()
{
   const char *pszXMLFile = strXMLFile_d->c_str ();

   try
   {
      cerr << "Parsing: " << pszXMLFile << endl;

      parser_d->parse (pszXMLFile);
   }
   catch (const XMLException& e)
   {
      cerr << "File not found: '" << pszXMLFile << "'" << endl;
      cerr << "Exception message is: " << XMLString::transcode (e.getMessage ()) << endl;
   }
}
