/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <fstream.h>

#include <util/XML4CDefs.hpp>
#include <util/XMLString.hpp>
#include <sax/ErrorHandler.hpp>

#include "MyErrorHandler.hpp"

MyErrorHandler::
MyErrorHandler (ReportError *pListener)
{
   pListener_d = pListener;
}

void MyErrorHandler::
warning (const SAXParseException& exception)
{
   cerr << "Warning at (file " << XMLString::transcode (exception.getSystemId ())
        << ", line " << exception.getLineNumber ()
        << ", char " << exception.getColumnNumber ()
        << "): " << XMLString::transcode (exception.getMessage ())
        << endl;
}

void MyErrorHandler::
error (const SAXParseException& exception)
{
   cerr << "Error at (file " << XMLString::transcode (exception.getSystemId ())
        << ", line " << exception.getLineNumber ()
        << ", char " << exception.getColumnNumber ()
        << "): " << XMLString::transcode (exception.getMessage ())
        << endl;

   pListener_d->setErrorCondition ();
}

void MyErrorHandler::
fatalError (const SAXParseException& exception)
{
   cerr << "Fatal Error at (file " << XMLString::transcode (exception.getSystemId ())
        << ", line " << exception.getLineNumber ()
        << ", char " << exception.getColumnNumber ()
        << "): " << XMLString::transcode (exception.getMessage ())
        << endl;

   pListener_d->setErrorCondition ();
}

void MyErrorHandler::
resetErrors ()
{
}
